@@ -1,7 +1,9 @@
 #pragma once
 
 #if MG_ARCH == MG_ARCH_PICOSDK
+#if !defined(MG_ENABLE_LWIP) || !MG_ENABLE_LWIP
 #include <errno.h>
+#endif
 #include <stdarg.h>
 #include <stdbool.h>
 #include <stdint.h>
@@ -16,9 +18,7 @@ int mkdir(const char *, mode_t);
 
 #if MG_OTA == MG_OTA_PICOSDK
 #include <hardware/flash.h>
-#if PICO_RP2040
 #include <pico/bootrom.h>
 #endif
-#endif
 
 #endif
