@@ -20,8 +20,8 @@
 
 #if defined(_MSC_VER) || defined(__cplusplus)
 // add restrict support
-#if (defined(_MSC_VER) && _MSC_VER >= 1900) || defined(__clang__) || \
-    defined(__GNUC__)
+#if ((defined(_MSC_VER) && _MSC_VER >= 1900) && !defined(__cplusplus)) || \
+    defined(__clang__) || defined(__GNUC__)
 #define restrict __restrict
 #else
 #define restrict
@@ -104,10 +104,12 @@ static void initialize_state(uint32_t state[CHACHA20_STATE_WORDS],
                              const uint8_t key[CHACHA20_KEY_SIZE],
                              const uint8_t nonce[CHACHA20_NONCE_SIZE],
                              uint32_t counter) {
+#if 0
 #ifdef static_assert
   static_assert(sizeof(uint32_t) == 4,
                 "We don't support systems that do not conform to standard of "
                 "uint32_t being exact 32bit wide");
+#endif
 #endif
   state[0] = 0x61707865;
   state[1] = 0x3320646e;
@@ -1300,14 +1302,14 @@ static PORTABLE_8439_DECL void poly1305_calculate_mac(
   poly1305_finish(&poly_ctx, mac);
 }
 
-#define PM(p) ((size_t) (p))
+#define MG_PM(p) ((size_t) (p))
 
 // pointers overlap if the smaller either ahead of the end,
 // or its end is before the start of the other
 //
 // s_size should be smaller or equal to b_size
-#define OVERLAPPING(s, s_size, b, b_size) \
-  (PM(s) < PM((b) + (b_size))) && (PM(b) < PM((s) + (s_size)))
+#define MG_OVERLAPPING(s, s_size, b, b_size) \
+  (MG_PM(s) < MG_PM((b) + (b_size))) && (MG_PM(b) < MG_PM((s) + (s_size)))
 
 PORTABLE_8439_DECL size_t mg_chacha20_poly1305_encrypt(
     uint8_t *restrict cipher_text, const uint8_t key[RFC_8439_KEY_SIZE],
@@ -1315,7 +1317,7 @@ PORTABLE_8439_DECL size_t mg_chacha20_poly1305_encrypt(
     size_t ad_size, const uint8_t *restrict plain_text,
     size_t plain_text_size) {
   size_t new_size = plain_text_size + RFC_8439_TAG_SIZE;
-  if (OVERLAPPING(plain_text, plain_text_size, cipher_text, new_size)) {
+  if (MG_OVERLAPPING(plain_text, plain_text_size, cipher_text, new_size)) {
     return (size_t) -1;
   }
   chacha20_xor_stream(cipher_text, plain_text, plain_text_size, key, nonce, 1);
@@ -1330,7 +1332,7 @@ PORTABLE_8439_DECL size_t mg_chacha20_poly1305_decrypt(
     const uint8_t *restrict cipher_text, size_t cipher_text_size) {
   // first we calculate the mac and see if it lines up, only then do we decrypt
   size_t actual_size = cipher_text_size - RFC_8439_TAG_SIZE;
-  if (OVERLAPPING(plain_text, actual_size, cipher_text, cipher_text_size)) {
+  if (MG_OVERLAPPING(plain_text, actual_size, cipher_text, cipher_text_size)) {
     return (size_t) -1;
   }
 
